package com.blogspot.revbingo.threed;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.JFrame;

public class ThreeDee extends JFrame {

	private Random r = new Random(); // used to generate random points
	ArrayList<SceneObject> scene = new ArrayList<SceneObject>();

	public static void main(String[] args) {
		new ThreeDee();
	}

	public ThreeDee() {
		final ThreeDeePanel panel = new ThreeDeePanel();

		scene = createScene();	// create random points and give them to
		panel.setScene(scene);	// the panel to render 
								
		panel.setBackground(Color.BLACK);
		panel.addMouseMotionListener(new MouseMotionAdapter() {

			private int oldX;
			private int oldY;

			private double radian = Math.PI / 180;

			public void mouseDragged(MouseEvent e) {
				int x = e.getX();
				int y = e.getY();
				int dx = x - oldX;
				int dy = y - oldY;
				oldX = x;
				oldY = y;

				double xangle = dx * radian;
				double yangle = dy * radian;
				for (SceneObject d : scene) {
					d.rotateZ(xangle);
					d.rotateX(yangle);
				}
				panel.repaint();
			}
		});
		this.add(panel);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setSize(800, 600);
		this.setVisible(true);
	}

	/**
	 * Creates the 3D scene as a series of drawable objects
	 * 
	 * @return ArrayList<SceneObject> - the scene to be drawn
	 */
	private ArrayList<SceneObject> createScene() {
		int STP = 60;
		double stp = Math.PI * 2 / STP;
		double radius = 100;
		ArrayList<SceneObject> scene = new ArrayList<SceneObject>();

		Point[][] points = new Point[STP][(STP / 2) + 1];
		for (int phi = 0; phi < STP; phi++) {
			for (int theta = 0; theta < (STP / 2) + 1; theta++) {
				points[phi][theta] = new Point(radius * Math.cos(stp * phi)
						* Math.sin(stp * theta), radius * Math.sin(stp * phi)
						* Math.sin(stp * theta), radius * Math.cos(stp * theta));
			}
		}

		for (int x = 0; x < STP; x++) {
			for (int y = 0; y < (STP / 2); y++) {
				int xx = (x + 1) % STP;
				int yy = (y + 1);
				Point p1 = points[x][y];
				Point p2 = points[xx][y];
				Point p3 = points[x][yy];
				Point p4 = points[xx][yy];
				Triangle t = new Triangle(p1.x, p1.y, p1.z, p2.x, p2.y, p2.z,
						p3.x, p3.y, p3.z);
				Triangle t2 = new Triangle(p2.x, p2.y, p2.z, p4.x, p4.y, p4.z,
						p3.x, p3.y, p3.z);
				scene.add(t);
				scene.add(t2);
			}
		}

		return scene;
	}

	/**
	 * Utility method to return a number somewhere between -size and size.
	 * 
	 * @param size
	 * @return
	 */
	private double random(int size) {
		return (double) r.nextInt(size) - (size / 2);
	}
}
